﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=8.5
@EndOfDesignText@
'Custom View class 
'#Event: ExampleEvent (Value As Int)
'#DesignerProperty: Key: BooleanExample, DisplayName: Boolean Example, FieldType: Boolean, DefaultValue: True, Description: Example of a boolean property.
'#DesignerProperty: Key: IntExample, DisplayName: Int Example, FieldType: Int, DefaultValue: 10, MinRange: 0, MaxRange: 100, Description: Note that MinRange and MaxRange are optional.
'#DesignerProperty: Key: StringWithListExample, DisplayName: String With List, FieldType: String, DefaultValue: Sunday, List: Sunday|Monday|Tuesday|Wednesday|Thursday|Friday|Saturday
'#DesignerProperty: Key: StringExample, DisplayName: String Example, FieldType: String, DefaultValue: Text
'#DesignerProperty: Key: ColorExample, DisplayName: Color Example, FieldType: Color, DefaultValue: 0xFFCFDCDC, Description: You can use the built-in color picker to find the color values.
'#DesignerProperty: Key: DefaultColorExample, DisplayName: Default Color Example, FieldType: Color, DefaultValue: Null, Description: Setting the default value to Null means that a nullable field will be displayed.
Sub Class_Globals
	Private mEventName As String 'ignore
	Private mCallBack As Object 'ignore
	Private mBase As Panel
	Private Const DefaultColorConstant As Int = -984833 'ignore
	
	Private IndSurface As AcceleratedSurface
	Private Const INDICATORCOLOUR_OFF As Int = Colors.LightGray
	Private Const INDICATORCOLOUR_ON As Int = Colors.Red
	Private Indicators(16) As Boolean
	Private IndicatorRects(16) As Rect
	Private TH As Thread				'ignore
	Private MsgList As List
	Private MsgLock As Lock
	Private Interrupt As Boolean
	Private mSeqr As MidiSequencer
End Sub

Public Sub Initialize (Callback As Object, EventName As String)
	mEventName = EventName
	mCallBack = Callback
	
	'Initialize the Accelerated Surface, we need this as there are too many Midi notes received to display on the Gui Thread.
	IndSurface.Initialize("Surface",True)
End Sub

Public Sub DesignerCreateView (Base As Panel, Lbl As Label, Props As Map)
	mBase = Base
	mBase.AddView(IndSurface,0,10dip,mBase.Width,mBase.Height)
	IndSurface.Color = Colors.Transparent
	
	TH.Initialise("TH")
	MsgList.Initialize
	MsgLock.Initialize(False)
	
	'Rects for indicators and Gui Labels for Channel note Message, channel numbers
	For i = 0 To 15
		Dim ILeft As Int = 2Dip + (i * 12Dip)
		Dim R As Rect
		R.Initialize(ILeft,2Dip,ILeft + 10Dip,12Dip)
		IndicatorRects(i) = R
		
		Dim NMLeft As Int = 2Dip + (i * 12Dip)
		Dim Lbl As Label
		Lbl.Initialize("")
		mBase.AddView(Lbl,NMLeft,0,10Dip,10Dip)
		Lbl.Text = i + 1
		Lbl.TextSize = 8
		Lbl.Gravity = Gravity.BOTTOM + Gravity.CENTER_HORIZONTAL
	Next

    
End Sub

Public Sub SetSeqr(Seqr As MidiSequencer)
	mSeqr = Seqr
	'Create a MidiReceiver and register it with the Sequencer so we can monitor the midi messages,
	'pass it this Module.
	Dim MDI As MidiDeviceInfo = MidiDevice_Static.RegisterReceiver("Monitor","Midi Monitor","Midi Monitor",Me)
	mSeqr.SetReceiver(MDI)
	
	'We need monitor to add a Metaevent listener so that it can sync the display to the tempo.
	mSeqr.AddMetaEventListener(Me,"SyncTempo")
	
End Sub

Public Sub GetBase As Panel
	Return mBase
End Sub

'This module is the receiver for midi messages from the MidiRealTimeSequencer, we just need to create a Send(msg() As Byte) sub to get them
'And register it with the sequencer (See main above)
Sub Send(Msg() As Byte)
	'Implement a lock so we don'g get the Update thread removing an item while we are adding one here.
	MsgLock.Wait
	'Add the next message to the queue
	MsgList.Add(Msg)
	MsgLock.Unlock
End Sub
Sub Update
	Dim Msg() As Byte
	Do While True
		'If there are messages to process
		Do While MsgList.Size > 0
			'Read the next one
			Msg = MsgList.Get(0)
			'Implement a lock so we don'g get the Gui thread adding an item while we are removing one here.
			MsgLock.Wait
			'Remove the message from the queue
			MsgList.RemoveAt(0)
			MsgLock.Unlock
			'Process the message
			Dim Command As Int = Bit.And(Msg(0),0xF0)
			Dim Channel As Int = Bit.And(Msg(0),0xF)
			'Check it's a message we are interested in (Note on 0x9n or note off 0x8n)
			If Command < 0XA0 Then
				Dim Velocity As Int = Bit.And(Msg(2),0xFF)
				'We are only recording note on messages
				If Command = 0x90 Then
					'With a velocity greater than 0
					If  Velocity > 0 Then
						Indicators(Channel) = True
					End If
				End If
			End If
			'Clear all indicators for AllSoundOff or AllNotesOff Messages
			If (Command = 0XB0 And Msg(1) = 120 Or Msg(1) = 123) Then
				ClearIndicators
			End If
			'Has the thread been asked to stop?
			If Interrupt Then Exit
		Loop
		If Interrupt Then
			'Clear indicators
			ClearIndicators
			Exit
		End If
	Loop
End Sub
Sub ClearIndicators
	For i = 0 To 15
		Indicators(i) = False
	Next
End Sub
'Draw the indicators on the accelerated surface
Sub Surface_Draw(AC As AS_Canvas)
	For i = 0 To Indicators.Length - 1
		If Indicators(i) Then
			AC.DrawOval(IndicatorRects(i),INDICATORCOLOUR_ON,True,0,False)
		Else
			AC.DrawOval(IndicatorRects(i),INDICATORCOLOUR_OFF,True,0,False)
		End If
	Next
	ClearIndicators
End Sub
'Start Monitoring
Sub Start
	'Try to sync the display update to the timing of a beat so we catch most of the events as they happen
	SyncTempo
	TH.Start(Me,"Update",Null)
End Sub
Sub Stop
	'Request the Update thread to stop
	Interrupt = True
End Sub
Sub TH_Ended(endedOK As Boolean, error As String) 'The thread has terminated. If endedOK is False error holds the reason for failure
	Log("TH Ended")
	'Update thread has stopped
	'Turn off the display update
	IndSurface.StopRegularDraw
	'Reset the interrupt
	Interrupt = False
End Sub
'This sub receives MetaMessages from the sequencer in response to out adding a metaEvent listener (Initialize sub)
Sub SyncTempo_MetaEvent(Event As MidiEvent)
	'Check if it's a tempo event
	If Event.Message.MetaType = MidiMessage_Static.META_TEMPO Then
		'Stop the display update
		IndSurface.StopRegularDraw
		SyncTempo
	End If
End Sub
'Sync the gui Update so it falls on a beat of the Sequence (or as near as possible)
Sub SyncTempo
	'Get the current tempo from the sequencer and start the drawing sub so it's updates coincide with a quarter note
	IndSurface.StartRegularDraw(mSeqr.GetTempoMPQ / 1000000 * mSeqr.GetSequence.Resolution)
End Sub